@@ -28,6 +28,8 @@ import hmac
 import logging
 from email.utils import formatdate
 from operator import itemgetter
+import functools
+
 from botocore.exceptions import NoCredentialsError
 from botocore.utils import normalize_url_path
 from botocore.compat import HTTPHeaders
@@ -38,6 +40,10 @@ logger = logging.getLogger(__name__)
 
 EMPTY_SHA256_HASH = (
     'e3b0c44298fc1c149afbf4c8996fb92427ae41e4649b934ca495991b7852b855')
+# This is the buffer size used when calculating sha256 checksums.
+# Experimenting with various buffer sizes showed that this value generally
+# gave the best result (in terms of performance).
+PAYLOAD_BUFFER = 1024 * 1024
 
 
 class BaseSigner(object):
@@ -54,8 +60,6 @@ class SigV2Auth(BaseSigner):
 
     def __init__(self, credentials):
         self.credentials = credentials
-        if self.credentials is None:
-            raise NoCredentialsError
 
     def calc_signature(self, request, params):
         logger.debug("Calculating signature using v2 auth.")
@@ -86,6 +90,8 @@ class SigV2Auth(BaseSigner):
         # Because of this we have to parse the query params
         # from the request body so we can update them with
         # the sigv2 auth params.
+        if self.credentials is None:
+            raise NoCredentialsError
         if request.data:
             # POST
             params = request.data
@@ -106,10 +112,10 @@ class SigV2Auth(BaseSigner):
 class SigV3Auth(BaseSigner):
     def __init__(self, credentials):
         self.credentials = credentials
-        if self.credentials is None:
-            raise NoCredentialsError
 
     def add_auth(self, request):
+        if self.credentials is None:
+            raise NoCredentialsError
         if 'Date' not in request.headers:
             request.headers['Date'] = formatdate(usegmt=True)
         if self.credentials.token:
@@ -132,8 +138,6 @@ class SigV4Auth(BaseSigner):
 
     def __init__(self, credentials, service_name, region_name):
         self.credentials = credentials
-        if self.credentials is None:
-            raise NoCredentialsError
         # We initialize these value here so the unit tests can have
         # valid values.  But these will get overriden in ``add_auth``
         # later for real requests.
@@ -165,17 +169,44 @@ class SigV4Auth(BaseSigner):
 
     def canonical_query_string(self, request):
         cqs = ''
+        # The query string can come from two parts.  One is the
+        # params attribute of the request.  The other is from the request
+        # url (in which case we have to re-split the url into its components
+        # and parse out the query string component).
         if request.params:
-            params = request.params
-            l = []
-            for param in params:
-                value = str(params[param])
-                l.append('%s=%s' % (quote(param, safe='-_.~'),
-                                    quote(value, safe='-_.~')))
-            l = sorted(l)
-            cqs = '&'.join(l)
+            return self._canonical_query_string_params(request.params)
+        else:
+            return self._canonical_query_string_url(urlsplit(request.url))
         return cqs
 
+    def _canonical_query_string_params(self, params):
+        l = []
+        for param in params:
+            value = str(params[param])
+            l.append('%s=%s' % (quote(param, safe='-_.~'),
+                                quote(value, safe='-_.~')))
+        l = sorted(l)
+        cqs = '&'.join(l)
+        return cqs
+
+    def _canonical_query_string_url(self, parts):
+        buf = ''
+        if parts.query:
+            qsa = parts.query.split('&')
+            qsa = [a.split('=', 1) for a in qsa]
+            quoted_qsa = []
+            for q in qsa:
+                if len(q) == 2:
+                    quoted_qsa.append(
+                        '%s=%s' % (quote(q[0], safe='-_.~'),
+                                   quote(unquote(q[1]), safe='-_.~')))
+                elif len(q) == 1:
+                    quoted_qsa.append('%s=' % quote(q[0], safe='-_.~'))
+            if len(quoted_qsa) > 0:
+                quoted_qsa.sort(key=itemgetter(0))
+                buf += '&'.join(quoted_qsa)
+        return buf
+
     def canonical_headers(self, headers_to_sign):
         """
         Return the headers that need to be included in the StringToSign
@@ -184,11 +215,11 @@ class SigV4Auth(BaseSigner):
         them into a string, separated by newlines.
         """
         headers = []
-        for key in set(headers_to_sign):
+        sorted_header_names = sorted(set(headers_to_sign))
+        for key in sorted_header_names:
             value = ','.join(v.strip() for v in
                              sorted(headers_to_sign.get_all(key)))
             headers.append('%s:%s' % (key, value))
-        headers.sort()
         return '\n'.join(headers)
 
     def signed_headers(self, headers_to_sign):
@@ -197,22 +228,39 @@ class SigV4Auth(BaseSigner):
         return ';'.join(l)
 
     def payload(self, request):
-        if request.body:
+        if request.body and hasattr(request.body, 'seek'):
+            position = request.body.tell()
+            read_chunksize = functools.partial(request.body.read,
+                                               PAYLOAD_BUFFER)
+            checksum = sha256()
+            for chunk in iter(read_chunksize, b''):
+                checksum.update(chunk)
+            hex_checksum = checksum.hexdigest()
+            request.body.seek(position)
+            return hex_checksum
+        elif request.body:
             return sha256(request.body.encode('utf-8')).hexdigest()
         else:
             return EMPTY_SHA256_HASH
 
     def canonical_request(self, request):
         cr = [request.method.upper()]
-        path = normalize_url_path(urlsplit(request.url).path)
+        path = self._normalize_url_path(urlsplit(request.url).path)
         cr.append(path)
         cr.append(self.canonical_query_string(request))
         headers_to_sign = self.headers_to_sign(request)
         cr.append(self.canonical_headers(headers_to_sign) + '\n')
         cr.append(self.signed_headers(headers_to_sign))
-        cr.append(self.payload(request))
+        if 'X-Amz-Content-SHA256' in request.headers:
+            body_checksum = request.headers['X-Amz-Content-SHA256']
+        else:
+            body_checksum = self.payload(request)
+        cr.append(body_checksum)
         return '\n'.join(cr)
 
+    def _normalize_url_path(self, path):
+        return normalize_url_path(path)
+
     def scope(self, args):
         scope = [self.credentials.access_key]
         scope.append(self.timestamp[0:8])
@@ -251,17 +299,14 @@ class SigV4Auth(BaseSigner):
         return self._sign(k_signing, string_to_sign, hex=True)
 
     def add_auth(self, request):
+        if self.credentials is None:
+            raise NoCredentialsError
         # Create a new timestamp for each signing event
         now = datetime.datetime.utcnow()
         self.timestamp = now.strftime('%Y%m%dT%H%M%SZ')
         # This could be a retry.  Make sure the previous
         # authorization header is removed first.
-        if 'Authorization' in request.headers:
-            del request.headers['Authorization']
-        if 'Date' not in request.headers:
-            request.headers['X-Amz-Date'] = self.timestamp
-        if self.credentials.token:
-            request.headers['X-Amz-Security-Token'] = self.credentials.token
+        self._add_headers_before_signing(request)
         canonical_request = self.canonical_request(request)
         logger.debug("Calculating signature using v4 auth.")
         logger.debug('CanonicalRequest:\n%s', canonical_request)
@@ -277,6 +322,25 @@ class SigV4Auth(BaseSigner):
         request.headers['Authorization'] = ', '.join(l)
         return request
 
+    def _add_headers_before_signing(self, request):
+        if 'Authorization' in request.headers:
+            del request.headers['Authorization']
+        if 'Date' not in request.headers:
+            request.headers['X-Amz-Date'] = self.timestamp
+        if self.credentials.token:
+            request.headers['X-Amz-Security-Token'] = self.credentials.token
+
+
+class S3SigV4Auth(SigV4Auth):
+
+    def _add_headers_before_signing(self, request):
+        super(S3SigV4Auth, self)._add_headers_before_signing(request)
+        request.headers['X-Amz-Content-SHA256'] = self.payload(request)
+
+    def _normalize_url_path(self, path):
+        # For S3, we do not normalize the path.
+        return path
+
 
 class HmacV1Auth(BaseSigner):
 
@@ -292,8 +356,6 @@ class HmacV1Auth(BaseSigner):
 
     def __init__(self, credentials, service_name=None, region_name=None):
         self.credentials = credentials
-        if self.credentials is None:
-            raise NoCredentialsError
         self.auth_path = None  # see comment in canonical_resource below
 
     def sign_string(self, string_to_sign):
@@ -386,6 +448,8 @@ class HmacV1Auth(BaseSigner):
         return self.sign_string(string_to_sign)
 
     def add_auth(self, request):
+        if self.credentials is None:
+            raise NoCredentialsError
         logger.debug("Calculating signature using hmacv1 auth.")
         split = urlsplit(request.url)
         logger.debug('HTTP request method: %s', request.method)
@@ -403,4 +467,5 @@ AUTH_TYPE_MAPS = {
     'v3': SigV3Auth,
     'v3https': SigV3Auth,
     's3': HmacV1Auth,
+    's3v4': S3SigV4Auth,
 }
