@@ -32,6 +32,17 @@ if six.PY3:
     from urllib.parse import urlunsplit
     from urllib.parse import urljoin
     from urllib.parse import parse_qsl
+    from io import IOBase as _IOBase
+    file_type = _IOBase
+
+    def set_socket_timeout(http_response, timeout):
+        """Set the timeout of the socket from an HTTPResponse.
+
+        :param http_response: An instance of ``httplib.HTTPResponse``
+
+        """
+        http_response._fp.fp.raw._sock.settimeout(timeout)
+
 else:
     from urllib import quote
     from urllib import unquote
@@ -40,6 +51,7 @@ else:
     from urlparse import urljoin
     from urlparse import parse_qsl
     from email.message import Message
+    file_type = file
     class HTTPHeaders(Message):
 
         # The __iter__ method is not available in python2.x, so we have
@@ -48,6 +60,14 @@ else:
             for field, value in self._headers:
                 yield field
 
+    def set_socket_timeout(http_response, timeout):
+        """Set the timeout of the socket from an HTTPResponse.
+
+        :param http_response: An instance of ``httplib.HTTPResponse``
+
+        """
+        http_response._fp.fp._sock.settimeout(timeout)
+
 try:
     from collections import OrderedDict
 except ImportError:
