@@ -217,3 +217,14 @@ class UnseekableStreamError(BotoCoreError):
     """
     fmt = ('Need to rewind the stream {stream_object}, but stream '
            'is not seekable.')
+
+
+class WaiterError(BotoCoreError):
+    """Waiter failed to reach desired state."""
+    fmt = 'Waiter {name} failed: {reason}'
+
+
+class IncompleteReadError(BotoCoreError):
+    """HTTP response did not return expected number of bytes."""
+    fmt = ('{actual_bytes} read, but total bytes '
+           'expected is {expected_bytes}.')
