@@ -79,7 +79,7 @@ class Parameter(BotoCoreObject):
             label = label.format(label=self.get_label())
         else:
             label = self.get_label()
-        built_params[label] = str(value)
+        built_params[label] = six.text_type(value)
 
     def build_parameter_query(self, value, built_params, label=''):
         value = self.validate(value)
@@ -374,9 +374,17 @@ class ListParameter(Parameter):
             else:
                 label = self.get_label()
             label = '%s.%s' % (label, 'member')
-        for i, v in enumerate(value, 1):
-            member_type.build_parameter_query(v, built_params,
-                                              '%s.%d' % (label, i))
+        if len(value) == 0 and self.required:
+            # If the parameter is required and an empty list is
+            # provided as a value, we should insert a parameter
+            # into the dictionary with the base name of the
+            # parameter and a value of the empty string. See
+            # ELB SetLoadBalancerPoliciesForBackendServer for example.
+            built_params[label.split('.')[0]] = ''
+        else:
+            for i, v in enumerate(value, 1):
+                member_type.build_parameter_query(v, built_params,
+                                                  '%s.%d' % (label, i))
 
     def build_parameter_json(self, value, built_params, label=''):
         value = self.validate(value)
@@ -415,13 +423,21 @@ class MapParameter(Parameter):
 
     def build_parameter_query(self, value, built_params, label=''):
         label = self.get_label()
+        if not self.flattened:
+            label = '%s.entry' % label
         key_type = self.keys
         member_type = self.members
         for i, v in enumerate(value, 1):
-            built_params['%s.%d.%s' % (label, i, key_type.xmlname)] = v
+            key_name = key_type.xmlname
+            if not key_name:
+                key_name = 'key'
+            built_params['%s.%d.%s' % (label, i, key_name)] = v
+            member_name = member_type.xmlname
+            if not member_name:
+                member_name = 'value'
             member_type.build_parameter_query(
                 value[v], built_params,
-                '%s.%d.%s' % (label, i, member_type.xmlname))
+                '%s.%d.%s' % (label, i, member_name))
 
     def build_parameter_json(self, value, built_params, label=''):
         label = self.get_label()
